require 'naught/null_class_builder/commands/define_explicit_conversions'
require 'naught/null_class_builder/commands/define_implicit_conversions'
require 'naught/null_class_builder/commands/pebble'
require 'naught/null_class_builder/commands/predicates_return'
require 'naught/null_class_builder/commands/singleton'
require 'naught/null_class_builder/commands/traceable'
require 'naught/null_class_builder/commands/mimic'
require 'naught/null_class_builder/commands/impersonate'
